using System;

using Syntriax.Engine.Core;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Serializers.Yaml;

public class ColorRGBAConverter : EngineTypeYamlSerializerBase<ColorRGBA>
{
    private static readonly int SUBSTRING_START_LENGTH = nameof(ColorRGBA).Length + 1;

    public override ColorRGBA Read(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        string value = parser.Consume<Scalar>().Value;
        string insideParenthesis = value[SUBSTRING_START_LENGTH..^1];
        string[] values = insideParenthesis.Split(", ");
        return new ColorRGBA(byte.Parse(values[0]), byte.Parse(values[1]), byte.Parse(values[2]), byte.Parse(values[3]));
    }

    public override void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        ColorRGBA rgb = (ColorRGBA)value!;
        emitter.Emit(new Scalar($"{nameof(ColorRGBA)}({rgb.R}, {rgb.G}, {rgb.B}, {rgb.A})"));
    }
}
